import { Component, ViewEncapsulation } from '@angular/core';

import { BusStop2Service } from './busStop2.service';
import { busStop2 } from './busStop2';
import { LocalDataSource } from 'ng2-smart-table';
import { GridOptions } from 'ag-grid/main';
import { BusStopCreateService } from '../../../../services/busstopcreate.service';
import { RegionSelectList } from '../../../../models/regionselectlist';
import { StateCommonSelectList } from '../../../../models/statecommonselectlist';
import { RoadSelectList } from '../../../../models/roadselectlist';
import { SettlementSelectList } from '../../../../models/settlementselectlist';
import { SurfaceTypeSelectList } from '../../../../models/surfacetypeselectlist';
import { EditorComponent } from '../../../../components/editor.component';
import { RendererComponent } from '../../../../components/renderer.component';

@Component({
  selector: 'statement-table',
  encapsulation: ViewEncapsulation.None,
  styleUrls: ['./busStop2.scss',
  '../../../../../../node_modules/ag-grid/dist/styles/ag-grid.css',
  '../../../../../../node_modules/ag-grid/dist/styles/theme-dark.css'],
  templateUrl: './busStop2.html',
})
export class BusStop2 {

  public showGrid: boolean;
  public rowData: any[] = [];
  public rowCount: string;
  public regions: RegionSelectList[];
  public states: StateCommonSelectList[];
  public surfaceTypes: SurfaceTypeSelectList[];
  public settlements: SettlementSelectList[];
  public roads: RoadSelectList[];
  private gridOptions: GridOptions;
  private columnDefs: any[];


  constructor(
    protected service: BusStop2Service,
    private dataService: BusStopCreateService,
    ) {
      this.gridOptions = <GridOptions>{};
      this.showGrid = true;
      this.gridOptions.rowModelType = 'virtual';
      this.dataService.getModels().then(models => {
        this.regions = models.regionSelectListDsM as RegionSelectList[];
        this.states = models.stateCommonSelectListDsM as StateCommonSelectList[];
        this.surfaceTypes = models.surfaceTypeSelectListDsM as SurfaceTypeSelectList[];
        this.settlements = models.settlementSelectListDsM as SettlementSelectList[];
        this.roads = models.roadSelectListDsM as RoadSelectList[];
      }).then(() => {
            this.createColumnDefs();
      });
      this.service.getData().then((data) => {
          if (data.length){
            this.rowData = data;
          } else {
            this.rowData = [new busStop2];
          }
      }).then(() => {
          this.gridOptions.api.setDatasource(this.setRowData(this.rowData));
          this.gridOptions.api.refreshVirtualPageCache();
      });
  }


  setRowData(allOfTheData) {
      let dataSource = {
          rowCount: null, // behave as infinite scroll
          getRows: function (params) {
              console.log('asking for ' + params.startRow + ' to ' + params.endRow);
              // At this point in your code, you would call the server, using $http if in AngularJS.
              // To make the demo look real, wait for 500ms before returning
              // take a slice of the total rows
              let rowsThisPage = allOfTheData.slice(params.startRow, params.endRow);
              // if on or after the last page, work out the last row.
              let lastRow = -1;
              if (allOfTheData.length <= params.endRow) {
                  lastRow = allOfTheData.length;
              }
              // call the success callback
              params.successCallback(rowsThisPage, lastRow);
          }
      };
      return dataSource;

  }
  onDeleteConfirm(event): void {
    if (window.confirm('Вы уверены что хотите удалить??')) {
      event.confirm.resolve();
    } else {
      event.confirm.reject();
    }
  }
  public addNewRow() {
        this.rowData.unshift(new busStop2());
        this.gridOptions.api.setDatasource(this.setRowData(this.rowData));
        this.gridOptions.api.refreshVirtualPageCache();
  }

  private createColumnDefs() {
    this.columnDefs = [
      {
        headerName: '#',
        width: 30,
        checkboxSelection: true,
        suppressSorting: true,
        suppressMenu: true,
        pinned: true
      },
      {
        headerName: 'ID',
        field: 'busStopId',
        width: 150
      },
      {
        headerName: 'Назва дороги',
        field: 'roadId',
        editable: true,
        width: 150,
        cellEditorFramework: EditorComponent,
        cellRendererFramework: RendererComponent,
        cellEditorParams: {
          data: this.roads,
          valueCol: 'roadId',
          labelCol: 'name'
        }
      },
      {
        headerName: 'Область',
        field: 'regionId',
        editable: true,
        width: 150,
        cellEditorFramework: EditorComponent,
        cellRendererFramework: RendererComponent,
        cellEditorParams: {
          data: this.regions,
          valueCol: 'regionId',
          labelCol: 'name'
        }
      },
      {
        headerName: 'Місцезнаходження, км+ справа',
        field: 'locationRight',
        editable: true,
        width: 150
      },
      {
        headerName: 'Місцезнаходження, км+ зліва',
        field: 'locationLeft',
        editable: true,
        width: 150
      },
      {
        headerName: 'Технічний стан',
        field: 'stateCommonId',
        editable: true,
        width: 150,
        cellEditorFramework: EditorComponent,
        cellRendererFramework: RendererComponent,
        cellEditorParams: {
          data: this.states,
          valueCol: 'stateCommonId',
          labelCol: 'value'
        }
      }
    ];
  }

    private onCellClicked($event) {
        console.log('onCellClicked: ' + $event.rowIndex + ' ' + $event.colDef.field);
    }

    private onCellValueChanged($event) {
        if ($event.oldValue !== $event.newValue) {
            let data = JSON.stringify($event.data);
            let id = $event.data.busStopId;
            let result = null;
            if (id) {
                result = this.service.update(id, data);
            } else {
                result = this.service.create(data);
            }
        }
    }

    private deleteRows() {
        let objects = this.gridOptions.api.getSelectedRows();
        if (!objects.length) {
            return ;
        }
        let ids = [];
        objects.forEach(element => {
            let id = element.busStopId;
            if (id) {
                ids.push(id);
            }
        });
        if (ids.length) {
            let rowData = this.rowData;
            rowData = rowData.filter(element => {
                let index = ids.indexOf(element.busStopId);
                if (index === -1) {
                    return true;
                } else {
                    this.service.delete(ids[index]);
                    return false;
                }
            });
            this.rowData = rowData;
            this.gridOptions.api.setDatasource(this.setRowData(this.rowData));
            this.gridOptions.api.refreshVirtualPageCache();
        }
    }

    private onCellDoubleClicked($event) {
        console.log('onCellDoubleClicked: ' + $event.rowIndex + ' ' + $event.colDef.field);
    }

    private onCellContextMenu($event) {
        console.log('onCellContextMenu: ' + $event.rowIndex + ' ' + $event.colDef.field);
    }

    private onCellFocused($event) {
        console.log('onCellFocused: (' + $event.rowIndex + ',' + $event.colIndex + ')');
    }

    private onRowSelected($event) {
        // taking out, as when we 'select all', it prints to much to the console!!
        // console.log('onRowSelected: ' + $event.node.data.name);
    }

    private onSelectionChanged() {
        console.log('selectionChanged');
    }

    private onBeforeFilterChanged() {
        console.log('beforeFilterChanged');
    }

    private onAfterFilterChanged() {
        console.log('afterFilterChanged');
    }

    private onFilterModified() {
        console.log('onFilterModified');
    }

    private onBeforeSortChanged() {
        console.log('onBeforeSortChanged');
    }

    private onAfterSortChanged() {
        console.log('onAfterSortChanged');
    }

    private onVirtualRowRemoved($event) {
        // because this event gets fired LOTS of times, we don't print it to the
        // console. if you want to see it, just uncomment out this line
        // console.log('onVirtualRowRemoved: ' + $event.rowIndex);
    }

    private onRowClicked($event) {
        console.log('onRowClicked: ' + $event.node.data.name);
    }

    public onQuickFilterChanged($event) {
        this.gridOptions.api.setQuickFilter($event.target.value);
    }

    // here we use one generic event to handle all the column type events.
    // the method just prints the event name
    private onColumnEvent($event) {
        console.log('onColumnEvent: ' + $event);
    }
}
